import * as wasm from './magnetic_pendulum_bg.wasm';

const lTextDecoder = typeof TextDecoder === 'undefined' ? (0, module.require)('util').TextDecoder : TextDecoder;

let cachedTextDecoder = new lTextDecoder('utf-8', { ignoreBOM: true, fatal: true });

cachedTextDecoder.decode();

let cachedUint8Memory0 = new Uint8Array();

function getUint8Memory0() {
    if (cachedUint8Memory0.byteLength === 0) {
        cachedUint8Memory0 = new Uint8Array(wasm.memory.buffer);
    }
    return cachedUint8Memory0;
}

function getStringFromWasm0(ptr, len) {
    return cachedTextDecoder.decode(getUint8Memory0().subarray(ptr, ptr + len));
}
/**
*/
export function greet() {
    wasm.greet();
}

function _assertClass(instance, klass) {
    if (!(instance instanceof klass)) {
        throw new Error(`expected instance of ${klass.name}`);
    }
    return instance.ptr;
}
/**
*/
export class Magnet {

    static __wrap(ptr) {
        const obj = Object.create(Magnet.prototype);
        obj.ptr = ptr;

        return obj;
    }

    __destroy_into_raw() {
        const ptr = this.ptr;
        this.ptr = 0;

        return ptr;
    }

    free() {
        const ptr = this.__destroy_into_raw();
        wasm.__wbg_magnet_free(ptr);
    }
    /**
    * @param {Vec2D} pos
    * @param {number} strength
    * @returns {Magnet}
    */
    static new(pos, strength) {
        _assertClass(pos, Vec2D);
        var ptr0 = pos.ptr;
        pos.ptr = 0;
        const ret = wasm.magnet_new(ptr0, strength);
        return Magnet.__wrap(ret);
    }
}
/**
*/
export class Pendulum {

    static __wrap(ptr) {
        const obj = Object.create(Pendulum.prototype);
        obj.ptr = ptr;

        return obj;
    }

    __destroy_into_raw() {
        const ptr = this.ptr;
        this.ptr = 0;

        return ptr;
    }

    free() {
        const ptr = this.__destroy_into_raw();
        wasm.__wbg_pendulum_free(ptr);
    }
    /**
    * @param {Vec2D} pos
    * @param {number} k
    * @param {number} friction
    * @returns {Pendulum}
    */
    static new(pos, k, friction) {
        _assertClass(pos, Vec2D);
        var ptr0 = pos.ptr;
        pos.ptr = 0;
        const ret = wasm.pendulum_new(ptr0, k, friction);
        return Pendulum.__wrap(ret);
    }
    /**
    * @returns {Vec2D}
    */
    pos() {
        const ret = wasm.pendulum_pos(this.ptr);
        return Vec2D.__wrap(ret);
    }
}
/**
*/
export class Universe {

    static __wrap(ptr) {
        const obj = Object.create(Universe.prototype);
        obj.ptr = ptr;

        return obj;
    }

    __destroy_into_raw() {
        const ptr = this.ptr;
        this.ptr = 0;

        return ptr;
    }

    free() {
        const ptr = this.__destroy_into_raw();
        wasm.__wbg_universe_free(ptr);
    }
    /**
    * @param {number} width
    * @param {number} height
    * @param {number} max_iters
    * @returns {Universe}
    */
    static new(width, height, max_iters) {
        const ret = wasm.universe_new(width, height, max_iters);
        return Universe.__wrap(ret);
    }
    /**
    * @param {Magnet} magnet
    */
    add_magnet(magnet) {
        _assertClass(magnet, Magnet);
        var ptr0 = magnet.ptr;
        magnet.ptr = 0;
        wasm.universe_add_magnet(this.ptr, ptr0);
    }
    /**
    */
    clear_magnets() {
        wasm.universe_clear_magnets(this.ptr);
    }
    /**
    * @param {Pendulum} pendulum
    */
    add_pendulum(pendulum) {
        _assertClass(pendulum, Pendulum);
        var ptr0 = pendulum.ptr;
        pendulum.ptr = 0;
        wasm.universe_add_pendulum(this.ptr, ptr0);
    }
    /**
    */
    clear_pendulums() {
        wasm.universe_clear_pendulums(this.ptr);
    }
    /**
    */
    tick() {
        wasm.universe_tick(this.ptr);
    }
}
/**
*/
export class Vec2D {

    static __wrap(ptr) {
        const obj = Object.create(Vec2D.prototype);
        obj.ptr = ptr;

        return obj;
    }

    __destroy_into_raw() {
        const ptr = this.ptr;
        this.ptr = 0;

        return ptr;
    }

    free() {
        const ptr = this.__destroy_into_raw();
        wasm.__wbg_vec2d_free(ptr);
    }
    /**
    * @returns {number}
    */
    get x() {
        const ret = wasm.__wbg_get_vec2d_x(this.ptr);
        return ret;
    }
    /**
    * @param {number} arg0
    */
    set x(arg0) {
        wasm.__wbg_set_vec2d_x(this.ptr, arg0);
    }
    /**
    * @returns {number}
    */
    get y() {
        const ret = wasm.__wbg_get_vec2d_y(this.ptr);
        return ret;
    }
    /**
    * @param {number} arg0
    */
    set y(arg0) {
        wasm.__wbg_set_vec2d_y(this.ptr, arg0);
    }
    /**
    * @param {number} x
    * @param {number} y
    * @returns {Vec2D}
    */
    static new(x, y) {
        const ret = wasm.vec2d_new(x, y);
        return Vec2D.__wrap(ret);
    }
    /**
    * @returns {Vec2D}
    */
    static zero() {
        const ret = wasm.vec2d_zero();
        return Vec2D.__wrap(ret);
    }
}

export function __wbg_alert_932cd10f98e7853f(arg0, arg1) {
    alert(getStringFromWasm0(arg0, arg1));
};

export function __wbindgen_throw(arg0, arg1) {
    throw new Error(getStringFromWasm0(arg0, arg1));
};

